from django.contrib import admin

from cached_hitcount.models import Hit,BlacklistIP

class HitAdmin(admin.ModelAdmin):
    list_display = ('__unicode__','hits','added')
    fields = ('hits',)


class BlacklistIPAdmin(admin.ModelAdmin):
    pass

admin.site.register(Hit, HitAdmin)

admin.site.register(BlacklistIP, BlacklistIPAdmin)




# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding index on 'Hit', fields ['added']
        db.create_index('cached_hitcount_hit', ['added'])


    def backwards(self, orm):
        # Removing index on 'Hit', fields ['added']
        db.delete_index('cached_hitcount_hit', ['added'])


    models = {
        'cached_hitcount.blacklistip': {
            'Meta': {'object_name': 'BlacklistIP'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '40'})
        },
        'cached_hitcount.hit': {
            'Meta': {'ordering': "('-hits',)", 'object_name': 'Hit'},
            'added': ('django.db.models.fields.DateField', [], {'default': 'datetime.datetime(2014, 3, 7, 0, 0)', 'db_index': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'content_type_set_for_hit'", 'to': "orm['contenttypes.ContentType']"}),
            'hits': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_pk': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['cached_hitcount']


from datetime import datetime, timedelta
import re

from django.core.cache import cache, get_cache
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q, Sum
from cached_hitcount.settings import CACHED_HITCOUNT_CACHE


# this is not intended to be an all-knowing IP address regex
IP_RE = re.compile('\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}')


def get_target_ctype_pk(obj):
    return ContentType.objects.get_for_model(obj), obj.pk


def return_period_from_string(arg):
    '''
    Takes a string such as "days=1,seconds=30" and strips the quotes
    and returns a dictionary with the key/value pairs
    '''
    period = {}

    if arg[0] == '"' and arg[-1] == '"':
        opt = arg[1:-1] #remove quotes
    else:
        opt = arg

    for o in opt.split(","):
        key, value = o.split("=")
        period[str(key)] = int(value)

    return period


def get_hit_count(*args, **kwargs):
    from cached_hitcount.models import Hit#avoid circular import


    hits =  Hit.objects.all()

    if "within" in kwargs:
        within = kwargs["within"]
        if not isinstance(within, timedelta):
            timedelta_dict = return_period_from_string(within)
            within = timedelta(**timedelta_dict)

        period = datetime.utcnow() - within
        hits = hits.filter(added__gte=period)

    if "object_ids" in kwargs and "object" in kwargs:
        object = kwargs["object"]#need object to get the content type
        object_ids = kwargs["object_ids"]
        content_type, object_pk = get_target_ctype_pk(object)
        hits =  hits.filter(object_pk__in=object_ids, content_type=content_type)
    elif "object" in kwargs:
        object = kwargs["object"]
        content_type, object_pk = get_target_ctype_pk(object)
        hits =  hits.filter(object_pk=object_pk, content_type=content_type)

    total_hits = 0

    try:
        total_hits = hits.aggregate(Sum('hits'))['hits__sum']
    except Hit.DoesNotExist:
        #create a Hit for this object as it doesn't exist yet
        Hit.objects.select_for_update().get_or_create(added=datetime.utcnow().date(), object_pk=object_pk, content_type=content_type)

    return str(total_hits)

def get_ip(request):
    """
    Retrieves the remote IP address from the request data.  If the user is
    behind a proxy, they may have a comma-separated list of IP addresses, so
    we need to account for that.  In such a case, only the first IP in the
    list will be retrieved.  Also, some hosts that use a proxy will put the
    REMOTE_ADDR into HTTP_X_FORWARDED_FOR.  This will handle pulling back the
    IP from the proper place.

    **NOTE** This function was taken from django-tracking (MIT LICENSE)
             http://code.google.com/p/django-tracking/
    """

    # if neither header contain a value, just use local loopback
    ip_address = request.META.get('HTTP_X_FORWARDED_FOR',
                                  request.META.get('REMOTE_ADDR', '127.0.0.1'))
    if ip_address:
        # make sure we have one and only one IP
        try:
            ip_address = IP_RE.match(ip_address)
            if ip_address:
                ip_address = ip_address.group(0)
            else:
                # no IP, probably from some dirty proxy or other device
                # throw in some bogus IP
                ip_address = '10.0.0.1'
        except IndexError:
            pass

    return ip_address

hitcount_cache = None
def get_hitcount_cache():
    global hitcount_cache
    if hitcount_cache is None:
        try:
            hitcount_cache = get_cache(CACHED_HITCOUNT_CACHE)
        except:
            # Use the default cache
            hitcount_cache = cache
    return hitcount_cache



import json

from django.http import Http404, HttpResponse, HttpResponseBadRequest

from django.conf import settings
from django.contrib.contenttypes.models import ContentType

from cached_hitcount.utils import get_ip, get_hitcount_cache
from cached_hitcount.models import BlacklistIP
from cached_hitcount.settings import CACHED_HITCOUNT_ENABLED, CACHED_HITCOUNT_CACHE_TIMEOUT, CACHED_HITCOUNT_EXCLUDE_IP_ADDRESS

def _update_hit_count(request, object_pk, ctype_pk):
    '''
    Evaluates a request's Hit and corresponding HitCount object and,
    after a bit of clever logic, either ignores the request or registers
    a new Hit.

    This is NOT a view!  But should be used within a view ...

    Returns True if the request was considered a Hit; returns False if not.
    '''
    ip = get_ip(request)

    # first, check our request against the blacklists before continuing
    if CACHED_HITCOUNT_EXCLUDE_IP_ADDRESS and ip in BlacklistIP.objects.get_cache():
        return False

    #save to memcache
    hitcount_cache = get_hitcount_cache()

    cache_key = "hitcount__%s__%s" % (ctype_pk, object_pk)
    try:
        hitcount_cache.incr(cache_key)
        return True
    except ValueError:#cache might have timed out
        count = 1
        hitcount_cache.set(cache_key, count, CACHED_HITCOUNT_CACHE_TIMEOUT)
        return True

    return False 

def json_error_response(error_message):
    return HttpResponse(json.dumps(dict(success=False,
                                              error_message=error_message)))


def update_hit_count_ajax(request):
    '''
    Ajax call that can be used to update a hit count.

    Ajax is not the only way to do this, but probably will cut down on 
    bots and spiders.

    See template tags for how to implement.
    '''

    # make sure this is an ajax request
    if not CACHED_HITCOUNT_ENABLED or not request.is_ajax():
        raise Http404()

    if request.method == "GET":
        return json_error_response("Hits counted via POST only.")

    object_pk = request.POST.get('object_pk', None)
    ctype_pk = request.POST.get('ctype_pk', None)
    
    status = "no hit recorded"
    if object_pk and ctype_pk:
        result = _update_hit_count(request, object_pk, ctype_pk)

        if result:
            status = "success"

    return HttpResponse(json.dumps({'status': status}),mimetype="application/json")



import os
from setuptools import setup, find_packages
from cached_hitcount import __version__

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "django-cached-hitcount",
    version = __version__,
    author = "Philip Roche",
    author_email = "phil@philroche,net",
    description = ("Basic app that allows you to track the number of hits/views for a particular object."),
    license = "GPLv3",
    keywords = "django hit count",
    url = "https://github.com/philroche/django-cached-hitcount",
    packages=find_packages(),
    long_description=read('README.md'),
    install_requires=[
        'Django>=1.4',
        'python-memcached-stats',#git+git://github.com/dlrust/python-memcached-stats.git
        'nexus',
        'django-jsonfield<0.9.4',
        'billiard',
        'python-dateutil',
        'gargoyle',
        'django-celery',
        'python-memcached',
        'south'
    ],
    dependency_links=[
        "http://github.com/dlrust/python-memcached-stats/tarball/master#egg=python-memcached-stats"
    ],
    include_package_data=True,
    zip_safe = False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from datetime import datetime, timedelta
from gargoyle import gargoyle
from celery.task import periodic_task
from memcached_stats import MemcachedStats

from django.core.cache import parse_backend_conf
from django.contrib.contenttypes.models import ContentType

from cached_hitcount.utils import get_hitcount_cache
from cached_hitcount.settings import CACHED_HITCOUNT_ENABLED, CACHED_HITCOUNT_CACHE, CACHED_HITCOUNT_CACHE_TIMEOUT, CACHED_HITCOUNT_IP_CACHE
from cached_hitcount.models import Hit
@periodic_task(run_every=timedelta(minutes=5))
def persist_hits():
    if CACHED_HITCOUNT_ENABLED:
        #TODO - add a try except around this n case it can't connect
        backend, location, params = parse_backend_conf(CACHED_HITCOUNT_CACHE)
        host, port = location.split(':')
        mem = MemcachedStats(host, port)
        keys = mem.keys()
        hitcount_cache = get_hitcount_cache()
        content_types = {}#used for keeping track of the content types so DB doesn't have to be queried each time
        for cache_key in keys:
            if "hitcount__" in cache_key and not CACHED_HITCOUNT_IP_CACHE in cache_key:
                cache_key = cache_key.split(':')[-1]#TODO why do we have to do this?
                count = hitcount_cache.get(cache_key)
                hitcount, ctype_pk, object_pk  = cache_key.split('__')

                if ctype_pk in content_types.keys():
                    content_type = content_types[ctype_pk]
                else:
                    content_type = ContentType.objects.get(id=ctype_pk)
                    content_types[ctype_pk] = content_type

                #save a new hit or increment this hits on an existing hit
                hit, created = Hit.objects.select_for_update().get_or_create(added=datetime.utcnow().date(), object_pk=object_pk, content_type=content_type)
                if hit and created:
                    hit.hits = count
                    hit.save()
                elif hit:
                    hit.hits = hit.hits + count
                    hit.save()

                #reset the hitcount for this object to 0
                hitcount_cache.set(cache_key, 0, CACHED_HITCOUNT_CACHE_TIMEOUT)








from django.conf import settings
from gargoyle import gargoyle

CACHED_HITCOUNT_CACHE = getattr(settings, 'HITCOUNT_CACHE', 'default')#The CACHES settings dict key to use for caching hits # hitcount_cache

CACHED_HITCOUNT_ENABLED = gargoyle.is_active('cached_hitcount', default=True) and hasattr(settings, 'CACHES') and CACHED_HITCOUNT_CACHE in getattr(settings, 'CACHES', {})

CACHED_HITCOUNT_CACHE_TIMEOUT = getattr(settings, 'HITCOUNT_CACHE_TIMEOUT', 600 )#600 = 10 mins,  3600 = 1 hour# How long in seconds should the hits be cached for before persisted
CACHED_HITCOUNT_IP_CACHE_TIMEOUT = getattr(settings, 'HITCOUNT_IP_CACHE_TIMEOUT', 86400 )#600 = 10 mins,  3600 = 1 hour, 86400=1 day# How long in seconds should the black list ip addresses be cached for
CACHED_HITCOUNT_PERSIST_SCHEDULE = getattr(settings, 'HITCOUNT_PERSIST_SCHEDULE', 15 )#minutes # How often should the hits be persisted
CACHED_HITCOUNT_IP_CACHE = getattr(settings, 'HITCOUNT_IP_CACHE', 'hitcount__blacklistip')#key name to use for caching the blacklist ip addresses
CACHED_HITCOUNT_EXCLUDE_IP_ADDRESS = getattr(settings, 'HITCOUNT_EXCLUDE_IP_ADDRESS', True)#Wheter or not you want to exclude some ip addresses
